﻿/**
 * Copyright © doubleSlash 2015
 * 
 * Visit us on http://www.doubleslash.de/
 * 
 * */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Office.Interop.Excel;

namespace MSExcelConditionalFormatting {
   class Program {

      static readonly object TYPE = Type.Missing;
      const string WORKBOOK_PATH = @"C:\Temp\MSExcelConditionalFormat.xlsx";

      Application excelApplication = null;
      Workbook newWorkbook = null;
      Worksheet targetSheet = null;

      static void Main(string[] args) {
         Console.WriteLine("---Start program---");

         Program program = new Program();
         program.PrepareExcelContent();
         program.ApplyFormatting();
         program.SaveContent();

         Console.WriteLine("---End program---");
         Console.ReadLine();
      }

      void PrepareExcelContent() {
         excelApplication = new Application();
         newWorkbook = excelApplication.Workbooks.Add(XlWBATemplate.xlWBATWorksheet);

         targetSheet = (Worksheet)(newWorkbook.Worksheets[1]);
         targetSheet.Name = "Conditional Formatting";
         targetSheet.get_Range("A1",
            TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 1);
         targetSheet.get_Range("A2", TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 2);
         targetSheet.get_Range("A3", TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 3);
         targetSheet.get_Range("A4", TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 4);
         targetSheet.get_Range("A5", TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 5);
         targetSheet.get_Range("A6", TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 6);
         targetSheet.get_Range("A7", TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 7);
         targetSheet.get_Range("A8", TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 8);
         targetSheet.get_Range("A9", TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 9);
         targetSheet.get_Range("A10", TYPE).set_Value(XlRangeValueDataType.xlRangeValueDefault, 10);
      }

      void ApplyFormatting() {
         IconSetCondition iconSetCondition = targetSheet.get_Range("A1:A10").FormatConditions.AddIconSetCondition();
         //Use traffic lights for this example
         iconSetCondition.IconSet = newWorkbook.IconSets[XlIconSet.xl3TrafficLights1];
         iconSetCondition.SetLastPriority();
         //Do not show content of cell
         iconSetCondition.ShowIconOnly = true;

         //Note: we never modify first icon.

         //Default: second icon is yellow for three traffic lights
         var yellowIcon = iconSetCondition.IconCriteria[2];
         yellowIcon.Type = XlConditionValueTypes.xlConditionValueNumber;
         yellowIcon.Value = 4;
         yellowIcon.Operator = (int)XlFormatConditionOperator.xlGreaterEqual;

         //Default: third icon is green for three traffic lights
         var greenIcon = iconSetCondition.IconCriteria[3];
         greenIcon.Type = XlConditionValueTypes.xlConditionValueNumber;
         greenIcon.Value = 7;
         greenIcon.Operator = (int)XlFormatConditionOperator.xlGreaterEqual;
      }

      void SaveContent() {
         try {
            //Suppress dialogs (e.g. for asking to overwrite existing file)
            excelApplication.DisplayAlerts = false;
            newWorkbook.SaveAs(WORKBOOK_PATH, TYPE, TYPE, TYPE, TYPE, TYPE, XlSaveAsAccessMode.xlNoChange, TYPE, TYPE, TYPE, TYPE, TYPE);

            Console.WriteLine("Applying conditional formatting on '" + WORKBOOK_PATH + "' successfully done.");

         } catch (Exception e) {
            Console.WriteLine("An error occured while writing into '" + WORKBOOK_PATH + "'. Message is: '" + e.Message + "'.");
         } finally {
            CleanUp();
         }
      }

      void CleanUp() {
         if (newWorkbook != null) {
            targetSheet = null;

            newWorkbook.Close(false, TYPE, TYPE);
            newWorkbook = null;
         }

         if (excelApplication != null) {
            excelApplication.Quit();
            excelApplication = null;
         }

         GC.Collect();
         GC.WaitForPendingFinalizers();
         GC.Collect();
         GC.WaitForPendingFinalizers();
      }
   }
}
